@@ -418,6 +418,8 @@ gupnp_service_info_get_udn (GUPnPServiceInfo *info)
  *
  * Get the UPnP service type, or %NULL.
  *
+ * Example: `urn:schemas-upnp-org:service:RenderingControl:1`
+ *
  * Returns: A constant string.
  **/
 const char *
@@ -442,7 +444,12 @@ gupnp_service_info_get_service_type (GUPnPServiceInfo *info)
  * gupnp_service_info_get_id:
  * @info: A #GUPnPServiceInfo
  *
- * Get the ID of this service, or %NULL if there is no ID.
+ * Get the serviceID of this service, or %NULL if there is no ID.
+ *
+ * The serviceID should be unique to a device. This makes it possible to provide
+ * the same serviceType multiple times on one device
+ *
+ * Example: `org:serviceId:RenderingControl`
  *
  * Return value: A string. This string should be freed with g_free() after use.
  **/
@@ -557,7 +564,11 @@ get_scpd_document_finished (GObject *source,
 
         gsize length;
         gconstpointer data = g_bytes_get_data (bytes, &length);
-        scpd = xmlRecoverMemory (data, length);
+        scpd = xmlReadMemory (data,
+                              length,
+                              NULL,
+                              NULL,
+                              XML_PARSE_NONET | XML_PARSE_RECOVER);
         if (scpd == NULL) {
                 g_task_return_new_error (task,
                                          GUPNP_SERVER_ERROR,
